@@ -1,4 +1,6 @@
 """Define a connection to the SimpliSafe websocket."""
+from dataclasses import InitVar, dataclass, field
+from datetime import datetime
 import logging
 from typing import Awaitable, Callable, Optional
 from urllib.parse import urlencode
@@ -6,7 +8,9 @@ from urllib.parse import urlencode
 from socketio import AsyncClient
 from socketio.exceptions import ConnectionError as ConnError, SocketIOError
 
+from simplipy.entity import EntityTypes
 from simplipy.errors import WebsocketError
+from simplipy.util.dt import utc_from_timestamp
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -33,6 +37,8 @@ EVENT_LOCK_ERROR = "lock_error"
 EVENT_LOCK_LOCKED = "lock_locked"
 EVENT_LOCK_UNLOCKED = "lock_unlocked"
 EVENT_MOTION_DETECTED = "motion_detected"
+EVENT_POWER_OUTAGE = "power_outage"
+EVENT_POWER_RESTORED = "power_restored"
 EVENT_SENSOR_NOT_RESPONDING = "sensor_not_responding"
 EVENT_SENSOR_RESTORED = "sensor_restored"
 
@@ -45,6 +51,7 @@ EVENT_MAPPING = {
     1159: EVENT_ALARM_TRIGGERED,
     1162: EVENT_ALARM_TRIGGERED,
     1170: EVENT_CAMERA_MOTION_DETECTED,
+    1301: EVENT_POWER_OUTAGE,
     1350: EVENT_CONNECTION_LOST,
     1381: EVENT_SENSOR_NOT_RESPONDING,
     1400: EVENT_DISARMED_BY_MASTER_PIN,
@@ -54,6 +61,7 @@ EVENT_MAPPING = {
     1429: EVENT_ENTRY_DETECTED,
     1458: EVENT_DOORBELL_DETECTED,
     1602: EVENT_AUTOMATIC_TEST,
+    3301: EVENT_POWER_RESTORED,
     3350: EVENT_CONNECTION_RESTORED,
     3381: EVENT_SENSOR_RESTORED,
     3401: EVENT_ARMED_AWAY_BY_KEYPAD,
@@ -71,52 +79,62 @@ EVENT_MAPPING = {
 }
 
 
-def get_event_type_from_payload(payload: dict) -> Optional[str]:
-    """Get the named websocket event from an event JSON payload.
+@dataclass(frozen=True)  # pylint: disable=too-many-instance-attributes
+class WebsocketEvent:
+    """Define a representation of a message."""
 
-    The ``payload`` parameter of this method should be the ``data`` parameter provided
-    to any function or coroutine that is passed to
-    :meth:`simplipy.websocket.Websocket.on_event`.
+    event_cid: InitVar[int]
+    message: str
+    system_id: int
+    timestamp: datetime
 
-    Returns one of the following:
-       * ``alarm_canceled``
-       * ``alarm_triggered``
-       * ``armed_away_by_keypad``
-       * ``armed_away_by_remote``
-       * ``armed_away``
-       * ``armed_home``
-       * ``automatic_test``
-       * ``away_exit_delay_by_keypad``
-       * ``away_exit_delay_by_remote``
-       * ``camera_motion_detected``
-       * ``connection_lost``
-       * ``connection_restored``
-       * ``disarmed_by_master_pin``
-       * ``disarmed_by_remote``
-       * ``doorbell_detected``
-       * ``entry_detected``
-       * ``home_exit_delay``
-       * ``lock_error``
-       * ``lock_locked``
-       * ``lock_unlocked``
-       * ``motion_detected``
-       * ``sensor_not_responding``
-       * ``sensor_restored``
+    event_type: Optional[str] = field(init=False)
 
-    :param payload: A event payload
-    :type payload: ``dict``
-    :rtype: ``str``
-    """
-    event_cid = payload["eventCid"]
-    if event_cid not in EVENT_MAPPING:
-        _LOGGER.warning(
-            'Encountered unknown websocket event type: %s ("%s"). Please report it at'
-            "https://github.com/bachya/simplisafe-python/issues.",
-            event_cid,
-            payload["info"],
-        )
-        return None
-    return EVENT_MAPPING[event_cid]
+    changed_by: Optional[str] = None
+    sensor_name: Optional[str] = None
+    sensor_serial: Optional[str] = None
+    sensor_type: Optional[EntityTypes] = None
+
+    def __post_init__(self, event_cid):
+        """Run post-init initialization."""
+        if event_cid in EVENT_MAPPING:
+            object.__setattr__(self, "event_type", EVENT_MAPPING[event_cid])
+        else:
+            _LOGGER.warning(
+                'Encountered unknown websocket event type: %s ("%s"). Please report it '
+                "at https://github.com/bachya/simplisafe-python/issues.",
+                event_cid,
+                self.message,
+            )
+            object.__setattr__(self, "event_type", None)
+
+        object.__setattr__(self, "timestamp", utc_from_timestamp(self.timestamp))
+
+        if self.sensor_type is not None:
+            try:
+                object.__setattr__(self, "sensor_type", EntityTypes(self.sensor_type))
+            except ValueError:
+                _LOGGER.warning(
+                    'Encountered unknown entity type: %s ("%s"). Please report it at'
+                    "https://github.com/home-assistant/home-assistant/issues.",
+                    self.sensor_type,
+                    self.message,
+                )
+                object.__setattr__(self, "sensor_type", None)
+
+
+def websocket_event_from_raw_data(event: dict):
+    """Create a Message object from a websocket event payload."""
+    return WebsocketEvent(
+        event["eventCid"],
+        event["info"],
+        event["sid"],
+        event["eventTimestamp"],
+        changed_by=event["pinName"],
+        sensor_name=event["sensorName"],
+        sensor_serial=event["sensorSerial"],
+        sensor_type=event["sensorType"],
+    )
 
 
 class Websocket:
@@ -193,7 +211,7 @@ class Websocket:
         """
         self._sync_disconnect_handler = target
 
-    def async_on_event(self, target: Callable[..., Awaitable]) -> None:
+    def async_on_event(self, target: Callable[..., Awaitable]) -> None:  # noqa: D202
         """Define a coroutine to be called an event is received.
 
         The couroutine will have a ``data`` parameter that contains the raw data from
@@ -202,9 +220,15 @@ class Websocket:
         :param target: A coroutine
         :type target: ``Callable[..., Awaitable]``
         """
-        self.on_event(target)
 
-    def on_event(self, target: Callable) -> None:
+        async def _async_on_event(event_data: dict):
+            """Act on the Message object."""
+            message = websocket_event_from_raw_data(event_data)
+            await target(message)
+
+        self._sio.on("event", _async_on_event, namespace=self._namespace)
+
+    def on_event(self, target: Callable) -> None:  # noqa: D202
         """Define a synchronous method to be called when an event is received.
 
         The method will have a ``data`` parameter that contains the raw data from the
@@ -213,4 +237,10 @@ class Websocket:
         :param target: A synchronous function
         :type target: ``Callable``
         """
-        self._sio.on("event", target, namespace=self._namespace)
+
+        def _on_event(event_data: dict):
+            """Act on the Message object."""
+            message = websocket_event_from_raw_data(event_data)
+            target(message)
+
+        self._sio.on("event", _on_event, namespace=self._namespace)
